@@ -24,6 +24,19 @@ static uint32_t _bounded_rand(uint32_t range) {
 	return m >> 32;
 }
 
+// https://prng.di.unimi.it/#remarks
+static double _uint64_to_double(uint64_t num) {
+	// A standard 64bit double floating-point number in IEEE floating point
+	// format has 52 bits of significand. Thus, the representation can actually
+	// store numbers with 53 significant binary digits.
+	double scale = 1.0 / (1ULL << 53);  // 1 divided by 2^53
+	double ret   = (num >> 11) * scale; // Top 53 bits divided by 1/2^53
+
+	//printf("Double: %0.15f\n", ret);
+
+	return ret;
+}
+
 // Use the C rand() function to return a 64 bit number
 static uint64_t _crand64() {
 	uint64_t high = rand();
@@ -33,7 +46,12 @@ static uint64_t _crand64() {
 	return ret;
 }
 
-// Borrowed from: https://elixir.bootlin.com/linux/v6.11.5/source/include/linux/hash.h
-static inline uint64_t _hash64(uint64_t val, uint8_t bits) {
-    return (val * 0x61c8864680b583ebull) >> (64 - bits);
+// MurmurHash3 Finalizer (Passes SmallCrush)
+static uint64_t _hash_mur3(uint64_t x) {
+    x ^= x >> 33;
+    x *= 0xff51afd7ed558ccd;
+    x ^= x >> 33;
+    x *= 0xc4ceb9fe1a85ec53;
+    x ^= x >> 33;
+    return x;
 }
